from ajenti.api import plugin
from ajenti.api.sensors import Sensor
from ajenti.plugins.dashboard.api import ConfigurableWidget
from ajenti.util import str_fsize


@plugin
class DiskSpaceWidget (ConfigurableWidget):
    name = _('Disk space')
    icon = 'hdd'

    def on_prepare(self):
        self.sensor = Sensor.find('disk-usage')
        self.append(self.ui.inflate('fstab:widget'))

    def on_start(self):
        self.find('device').text = self.config['device']
        u, t = self.sensor.value(self.config['device'])
        self.find('percent').text = str_fsize(u)
        self.find('usage').value = float(1.0 * u / t)

    def create_config(self):
        return {'device': ''}

    def on_config_start(self):
        device_list = self.dialog.find('device')
        lst = self.sensor.get_variants()
        device_list.labels = lst
        device_list.values = lst
        device_list.value = self.config['device']

    def on_config_save(self):
        self.config['device'] = self.dialog.find('device').value



from ajenti.api import *
from ajenti.ui import UIElement, p


@p('config', public=False, doc="current configuration dict of this widget instance")
@p('container', type=int, default=0)
@p('index', type=int, default=0)
@interface
class DashboardWidget (BasePlugin, UIElement):
    """
    Base class for widgets (inherits :class:`ajenti.ui.UIElement`).
    """
    typeid = 'dashboard:widget'
    name = '---'
    """ Widget type name """
    icon = None
    """ Widget icon name """
    hidden = False
    """ If True, user will not be able to add this widget through dashboard """

    def save_config(self):
        self.event('save-config')


@interface
class ConfigurableWidget (DashboardWidget):
    """
    Base class for widgets with a configuration dialog
    """
    def init(self):
        self.on_prepare()
        self.dialog = self.find('config-dialog')
        if not self.config and self.dialog:
            self.config = self.create_config()
            self.on_config_start()
            self.dialog.on('button', self.on_config)
            self.dialog.visible = True
        else:
            self.on_start()

    def on_config(self, button):
        self.dialog.visible = False
        self.on_config_save()
        self.save_config()

    def on_prepare(self):
        """
        Widget should create its UI in this method. Called before **self.config** is created
        """

    def on_start(self):
        """
        Widget should populate its UI in this method. **self.config** is now available.
        """

    def create_config(self):
        """
        Should return a default config dict
        """
        return None

    def on_config_start(self):
        """
        Called when user begins to configure the widget. Should populate the config dialog.
        """

    def on_config_save(self):
        """
        Called when user is done configuring the widget.
        """



from ajenti.api import *


class RAIDDevice (object):
    def __init__(self):
        self.name = ''
        self.up = False
        self.failed = False
        self.index = 0


class RAIDArray (object):
    def __init__(self):
        self.name = ''
        self.type = ''
        self.blocks = 0
        self.metadata = ''
        self.devices = []
        self.recovery = False
        self.recovery_progress = 0
        self.recovery_remaining = ''


@plugin
class RAIDManager (BasePlugin):
    def __init__(self):
        self.refresh()

    def refresh(self):
        self.arrays = []
        ll = open('/proc/mdstat').read().splitlines()
        while ll:
            l = ll.pop(0)
            if l.startswith('Personalities'):
                continue
            if l.startswith('unused'):
                continue
            if ':' in l:
                array = RAIDArray()
                array.name, tokens = l.split(':')
                array.name = array.name.strip()
                tokens = tokens.split()
                array.state = tokens[0]
                array.active = array.state == 'active'
                array.type = tokens[1]
                devices = tokens[2:]

                l = ll.pop(0)
                tokens = l.split()
                array.blocks = int(tokens[0])
                array.metadata = tokens[3]

                states = tokens[-1][1:-1]

                self.arrays.append(array)

                for i in range(len(devices)):
                    device = RAIDDevice()
                    device.name = devices[i].split('[')[0]
                    device.index = int(devices[i].split('[')[1].split(']')[0])
                    device.up = states[i] == 'U'
                    array.devices.append(device)

                l = ll.pop(0)
                if 'recovery' in l:
                    array.recovery = True
                    array.recovery_progress = float(l.split()[3].strip('%')) / 100
                    array.recovery_remaining = l.split()[5].split('=')[1]



import subprocess

from ajenti.api import *
from ajenti.plugins.configurator.api import ClassConfigEditor
from ajenti.plugins.db_common.api import DBPlugin, Database, User


@plugin
class MySQLClassConfigEditor (ClassConfigEditor):
    title = 'MySQL'
    icon = 'table'

    def init(self):
        self.append(self.ui.inflate('mysql:config'))


@plugin
class MySQLPlugin (DBPlugin):
    default_classconfig = {'user': 'root', 'password': ''}
    classconfig_editor = MySQLClassConfigEditor

    service_name = 'mysql'
    service_buttons = [
        {
            'command': 'reload',
            'text': _('Reload'),
            'icon': 'step-forward',
        }
    ]

    def init(self):
        self.title = 'MySQL'
        self.category = _('Software')
        self.icon = 'table'

    def query(self, sql, db=''):
        p = subprocess.Popen([
            'mysql',
            '-u' + self.classconfig['user'],
            '-p' + self.classconfig['password']],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        o, e = p.communicate((('USE %s; ' % db) if db else '') + sql)
        if p.returncode:
            raise Exception(e)
        return filter(None, o.splitlines()[1:])

    def query_sql(self, db, sql):
        r = []
        for l in self.query(sql + ';', db):
            r.append(l.split('\t'))
        return r

    def query_databases(self):
        r = []
        for l in self.query('SHOW DATABASES;'):
            db = Database()
            db.name = l
            r.append(db)
        return r

    def query_drop(self, db):
        self.query("DROP DATABASE `%s`;" % db.name)

    def query_create(self, name):
        self.query("CREATE DATABASE `%s` CHARACTER SET UTF8;" % name)

    def query_users(self):
        r = []
        for l in self.query('USE mysql; SELECT * FROM user;'):
            u = User()
            u.host, u.name = l.split('\t')[:2]
            r.append(u)
        return r

    def query_create_user(self, user):
        self.query("CREATE USER `%s`@`%s` IDENTIFIED BY '%s'" % (user.name, user.host, user.password))

    def query_drop_user(self, user):
        self.query("DROP USER `%s`@`%s`" % (user.name, user.host))



import os
import subprocess

import ajenti
from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder

from reconfigure.configs import CTDBConfig, CTDBNodesConfig, CTDBPublicAddressesConfig
from reconfigure.items.ctdb import NodeData, PublicAddressData


@plugin
class CTDB (SectionPlugin):
    nodes_file = '/etc/ctdb/nodes'
    addresses_file = '/etc/ctdb/public_addresses'

    def init(self):
        self.title = _('Samba Cluster')
        self.icon = 'folder-close'
        self.category = _('Software')

        self.append(self.ui.inflate('ctdb:main'))

        self.config_path = {
            'debian': '/etc/default/ctdb',
            'centos': '/etc/sysconfig/ctdb'
        }[ajenti.platform]

        self.config = CTDBConfig(path=self.config_path)
        self.config.load()

        self.binder = Binder(None, self.find('main-config'))
        self.n_binder = Binder(None, self.find('nodes-config'))
        self.a_binder = Binder(None, self.find('addresses-config'))
        self.find('nodes').new_item = lambda c: NodeData()
        self.find('addresses').new_item = lambda c: PublicAddressData()

    def on_page_load(self):
        n_path = self.config.tree.nodes_file
        self.nodes_config = CTDBNodesConfig(path=n_path)
        if not os.path.exists(n_path):
            open(n_path, 'w').close()
        self.nodes_config.load()

        a_path = self.config.tree.public_addresses_file
        self.addresses_config = CTDBPublicAddressesConfig(path=a_path)
        if not os.path.exists(a_path):
            open(a_path, 'w').close()
        self.addresses_config.load()

        self.config.load()
        self.binder.reset(self.config.tree).autodiscover().populate()
        self.n_binder.reset(self.nodes_config.tree).autodiscover().populate()
        self.a_binder.reset(self.addresses_config.tree).autodiscover().populate()
        self.refresh()

    @on('refresh', 'click')
    def refresh(self):
        try:
            self.find('status').value = subprocess.check_output(['ctdb', 'status'])
            self.find('status-ip').value = subprocess.check_output(['ctdb', 'ip'])
        except:
            self.find('status').value = _('Failed to obtain CTDB status')

    @on('save', 'click')
    def save(self):
        self.binder.update()
        self.n_binder.update()
        self.a_binder.update()
        self.config.save()
        self.nodes_config.save()
        self.addresses_config.save()
        self.context.notify('info', _('Saved'))



import logging
import sys
from datetime import datetime


class DebugHandler (logging.StreamHandler):
    """
    Captures log into a buffer for error reports
    """

    def __init__(self):
        self.capturing = False
        self.buffer = ''

    def start(self):
        self.capturing = True

    def stop(self):
        self.capturing = False

    def handle(self, record):
        if self.capturing:
            self.buffer += self.formatter.format(record) + '\n'


class ConsoleHandler (logging.StreamHandler):
    def __init__(self, stream, debug):
        self.debug = debug
        logging.StreamHandler.__init__(self, stream)

    def handle(self, record):
        if not self.stream.isatty():
            return logging.StreamHandler.handle(self, record)

        s = ''
        d = datetime.fromtimestamp(record.created)
        s += d.strftime("\033[37m%d.%m.%Y %H:%M \033[0m")
        if self.debug:
            s += ('%s:%s' % (record.filename, record.lineno)).ljust(30)
        l = ''
        if record.levelname == 'DEBUG':
            l = '\033[37mDEBUG\033[0m '
        if record.levelname == 'INFO':
            l = '\033[32mINFO\033[0m  '
        if record.levelname == 'WARNING':
            l = '\033[33mWARN\033[0m  '
        if record.levelname == 'ERROR':
            l = '\033[31mERROR\033[0m '
        s += l.ljust(9)
        try:
            s += record.msg % record.args
        except:
            s += record.msg
        s += '\n'
        self.stream.write(s)


def make_log(debug=False, log_level=logging.INFO):
    log = logging.getLogger()
    log.setLevel(logging.DEBUG)

    stdout = ConsoleHandler(sys.stdout, debug)
    stdout.setLevel(log_level)

    logging.blackbox = DebugHandler()
    logging.blackbox.setLevel(logging.DEBUG)
    dformatter = logging.Formatter('%(asctime)s %(levelname)-8s %(module)s.%(funcName)s(): %(message)s')
    logging.blackbox.setFormatter(dformatter)
    stdout.setFormatter(dformatter)
    log.addHandler(logging.blackbox)

    log.addHandler(stdout)

    return log


def init(level=logging.INFO):
    make_log(debug=level == logging.DEBUG, log_level=level)
    logging.blackbox.start()



import os

from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder


class AvailabilitySymlinks (object):
    """
    Manage directories of following style::

        --sites.available
         |-a.site
         --b.site
        --sites.enabled
         --a.site -> ../sites.available/a.site
    """

    def __init__(self, dir_a, dir_e):
        self.dir_a, self.dir_e = dir_a, dir_e

    def list_available(self):
        return os.listdir(self.dir_a)

    def is_enabled(self, entry):
        return self.find_link(entry) is not None

    def get_path(self, entry):
        return os.path.abspath(os.path.join(self.dir_a, entry))

    def find_link(self, entry):
        path = self.get_path(entry)

        for e in os.listdir(self.dir_e):
            if os.path.abspath(os.path.realpath(os.path.join(self.dir_e, e))) == path:
                return e

    def enable(self, entry):
        e = self.find_link(entry)
        if not e:
            os.symlink(self.get_path(entry), os.path.join(self.dir_e, entry))

    def disable(self, entry):
        e = self.find_link(entry)
        if e:
            os.unlink(os.path.join(self.dir_e, e))

    def rename(self, old, new):
        on = self.is_enabled(old)
        self.disable(old)
        os.rename(self.get_path(old), self.get_path(new))
        if on:
            self.enable(new)

    def delete(self, entry):
        self.disable(entry)
        os.unlink(self.get_path(entry))

    def open(self, entry, mode='r'):
        return open(os.path.join(self.dir_a, entry), mode)

    def exists(self):
        return os.path.exists(self.dir_a) and os.path.exists(self.dir_e)


class WebserverHost (object):
    def __init__(self, dir, entry):
        self.name = entry
        self.dir = dir
        self.active = dir.is_enabled(entry)
        self.config = dir.open(entry).read()

    def save(self):
        self.dir.open(self.name, 'w').write(self.config)
        if self.active:
            self.dir.enable(self.name)
        else:
            self.dir.disable(self.name)


class WebserverPlugin (SectionPlugin):
    service_name = ''
    service_buttons = []
    hosts_available_dir = ''
    hosts_enabled_dir = ''
    template = ''

    def init(self):
        self.append(self.ui.inflate('webserver_common:main'))
        self.binder = Binder(None, self)
        self.find_type('servicebar').buttons = self.service_buttons
        self.hosts_dir = AvailabilitySymlinks(self.hosts_available_dir, self.hosts_enabled_dir)

        def delete_host(host, c):
            c.remove(host)
            self.hosts_dir.delete(host.name)

        def on_host_bind(o, c, host, u):
            host.__old_name = host.name

        def on_host_update(o, c, host, u):
            if host.__old_name != host.name:
                self.hosts_dir.rename(host.__old_name, host.name)
            host.save()

        def new_host(c):
            name = 'untitled'
            self.hosts_dir.open(name, 'w').write(self.template)
            return WebserverHost(self.hosts_dir, name)

        self.find('hosts').delete_item = delete_host
        self.find('hosts').new_item = new_host
        self.find('hosts').post_item_bind = on_host_bind
        self.find('hosts').post_item_update = on_host_update

    def on_page_load(self):
        self.refresh()

    @on('save-button', 'click')
    def save(self):
        self.binder.update()
        self.refresh()
        self.context.notify('info', 'Saved')

    def refresh(self):
        self.hosts = [WebserverHost(self.hosts_dir, x) for x in self.hosts_dir.list_available()]
        self.binder.reset(self).autodiscover().populate()
        self.find_type('servicebar').reload()



import os
import subprocess

import ajenti
from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin, intent
from ajenti.ui import on
from ajenti.ui.binder import Binder, CollectionAutoBinding, DictAutoBinding
from ajenti.users import UserManager, PermissionProvider, restrict
from reconfigure.items.ajenti import UserData


@plugin
class ClassConfigManager (BasePlugin):
    def init(self):
        self.classes = []

    def reload(self):
        if self.context.user.name == 'root':
            self.classes = BasePlugin.get_instances(self.context)
            self.classes += BasePlugin.get_instances(self.context.parent)
        else:
            self.classes = filter(lambda x: not x.classconfig_root, BasePlugin.get_instances())
        self.classes = filter(lambda x: x.classconfig_editor, self.classes)
        self.classes = list(set(self.classes))


@plugin
class Configurator (SectionPlugin):
    def init(self):
        self.title = _('Configure')
        self.icon = 'wrench'
        self.category = ''
        self.order = 50

        self.append(self.ui.inflate('configurator:main'))

        self.binder = Binder(ajenti.config.tree, self.find('ajenti-config'))

        self.ccmgr = ClassConfigManager.get()
        self.classconfig_binding = Binder(self.ccmgr, self.find('classconfigs'))
        self.classconfig_rows = {}

        def post_classconfig_bind(object, collection, item, ui):
            self.classconfig_rows[item] = ui
            editor = item.classconfig_editor.new(self.ui)
            ui.find('container').append(editor)
            binder = DictAutoBinding(item, 'classconfig', editor.find('bind'))
            binder.populate()

            def save():
                binder.update()
                item.save_classconfig()
                self.context.notify('info', _('Saved'))

            ui.find('save').on('click', save)

        self.find('classconfigs').find('classes').post_item_bind = post_classconfig_bind

        self.find('users').new_item = lambda c: UserData()

        def post_user_bind(object, collection, item, ui):
            box = ui.find('permissions')
            box.empty()
            ui.find('name-edit').visible = item.name != 'root'
            ui.find('name-label').visible = item.name == 'root'
            for prov in PermissionProvider.get_all():
                line = self.ui.create('tab', title=prov.get_name())
                box.append(line)
                for perm in prov.get_permissions():
                    line.append(
                        self.ui.create('checkbox', id=perm[0], text=perm[1], value=(perm[0] in item.permissions))
                    )
        self.find('users').post_item_bind = post_user_bind

        def post_user_update(object, collection, item, ui):
            box = ui.find('permissions')
            for prov in PermissionProvider.get_all():
                for perm in prov.get_permissions():
                    has = box.find(perm[0]).value
                    if has and not perm[0] in item.permissions:
                        item.permissions.append(perm[0])
                    if not has and perm[0] in item.permissions:
                        item.permissions.remove(perm[0])
            if ui.find('password').value:
                item.password = ui.find('password').value
        self.find('users').post_item_update = post_user_update

        self.refresh()

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.binder.reset().autodiscover().populate()
        self.ccmgr.reload()
        self.classconfig_binding.reset().autodiscover().populate()

    @on('save-button', 'click')
    @restrict('configurator:configure')
    def save(self):
        self.binder.update()
        for user in ajenti.config.tree.users.values():
            if not '|' in user.password:
                user.password = UserManager.get().hash_password(user.password)
        self.binder.populate()
        ajenti.config.save()
        self.context.notify('info', _('Saved. Please restart Ajenti for changes to take effect.'))

    @on('fake-ssl', 'click')
    def on_gen_ssl(self):
        host = self.find('fake-ssl-host').value
        if host == '':
            self.context.notify('error', _('Please supply hostname'))
        else:
            self.gen_ssl(host)

    @on('restart-button', 'click')
    def on_restart(self):
        ajenti.restart()

    @intent('configure-plugin')
    def configure_plugin(self, plugin=None):
        self.find('tabs').active = 1
        self.refresh()
        #if plugin in self.classconfig_rows:
        #    self.classconfig_rows[plugin].children[0].expanded = True
        #    print self.classconfig_rows[plugin].children[0]
        if plugin:
            self.context.notify('info', _('Please configure %s plugin!') % plugin.classconfig_editor.title)
        self.activate()

    @intent('setup-fake-ssl')
    def gen_ssl(self, host):
        self.save()
        subprocess.call(['ajenti-ssl-gen', host, '-f'])
        ajenti.config.load()
        self.refresh()


@plugin
class ConfigurationPermissionsProvider (PermissionProvider):
    def get_name(self):
        return _('Configuration')

    def get_permissions(self):
        return [
            ('configurator:configure', _('Modify Ajenti configuration')),
            ('configurator:restart', _('Restart Ajenti')),
        ]



from ajenti.ui import *
from ajenti.ui.binder import Binder

from api import *


@plugin
class LinuxIfUpDownNetworkConfigSet (NetworkConfigBit):
    cls = 'linux-ifupdown'
    title = 'Scripts'

    def init(self):
        self.append(self.ui.inflate('network:bit-linux-ifupdown'))
        self.binder = Binder(self.iface, self)

    def refresh(self):
        self.binder.reset(self.iface).autodiscover().populate()

    def apply(self):
        self.binder.update()



import subprocess

from ajenti.api import *
from ajenti.plugins.db_common.api import DBPlugin, Database, User


@plugin
class PSQLPlugin (DBPlugin):
    service_name = 'postgresql'
    service_buttons = [
        {
            'command': 'reload',
            'text': _('Reload'),
            'icon': 'step-forward',
        }
    ]

    def init(self):
        self.title = 'PostgreSQL'
        self.category = _('Software')
        self.icon = 'table'

    def query(self, sql, db=''):
        p = subprocess.Popen([
            'su',
            'postgres',
            '-c',
            'psql -R"~~~" -A -t -c "%s" %s' % (sql, db)],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        o, e = p.communicate()
        if p.returncode:
            raise Exception(e)
        return filter(None, o.split('~~~'))

    def query_sql(self, db, sql):
        r = []
        for l in self.query(sql.replace('"', '\\"') + ';', db):
            r.append(l.split('|'))
        return r

    def query_databases(self):
        r = []
        for l in self.query('\\l'):
            db = Database()
            db.name = l.split('|')[0]
            r.append(db)
        return r

    def query_drop(self, db):
        self.query('DROP DATABASE %s;' % db.name)

    def query_create(self, name):
        self.query('CREATE DATABASE %s;' % name)

    def query_users(self):
        r = []
        for l in self.query('\\du'):
            u = User()
            u.host, u.name = '', l.split('|')[0]
            r.append(u)
        return r

    def query_create_user(self, user):
        self.query('CREATE USER %s WITH PASSWORD \'%s\';' % (user.name, user.password))

    def query_drop_user(self, user):
        self.query('DROP USER "%s";' % user.name)
